package de.intevation.flys.client.server;

import java.util.ArrayList;
import java.util.List;

import javax.xml.xpath.XPathConstants;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;

import org.apache.log4j.Logger;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

import de.intevation.artifacts.common.ArtifactNamespaceContext;
import de.intevation.artifacts.common.utils.XMLUtils;

import de.intevation.artifacts.httpclient.exceptions.ConnectionException;
import de.intevation.artifacts.httpclient.http.HttpClient;
import de.intevation.artifacts.httpclient.http.HttpClientImpl;

import de.intevation.flys.client.shared.exceptions.ServerException;
import de.intevation.flys.client.shared.model.DefaultRiver;
import de.intevation.flys.client.shared.model.River;
import de.intevation.flys.client.client.services.RiverService;


/**
 * This interface provides a method to list the supported rivers of the artifact
 * server.
 *
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class RiverServiceImpl
extends      RemoteServiceServlet
implements   RiverService
{
    /** Private logger. */
    private static final Logger logger =
        Logger.getLogger(RiverServiceImpl.class);

    /** The XPath string that points to the rivers in the resulting document.*/
    public static final String XPATH_RIVERS = "/art:rivers/art:river";

    /** The error message key that is thrown if an error occured while reading
     * the supported rivers from server.*/
    public static final String ERROR_NO_RIVERS_FOUND = "error_no_rivers_found";


    /** Get river list. */
    public River[] list(String locale)
    throws ServerException
    {
        String url = getServletContext().getInitParameter("server-url");

        Document doc = XMLUtils.newDocument();

        XMLUtils.ElementCreator ec = new XMLUtils.ElementCreator(
            doc,
            ArtifactNamespaceContext.NAMESPACE_URI,
            ArtifactNamespaceContext.NAMESPACE_PREFIX);

        doc.appendChild(ec.create("action"));

        HttpClient client = new HttpClientImpl(url, locale);

        try {
            Document res = client.callService(url, "rivers", doc);

            NodeList rivers = (NodeList) XMLUtils.xpath(
                res,
                XPATH_RIVERS,
                XPathConstants.NODESET,
                ArtifactNamespaceContext.INSTANCE);

            if (rivers == null || rivers.getLength() == 0) {
                throw new ServerException(ERROR_NO_RIVERS_FOUND);
            }

            int count = rivers.getLength();

            List<River> theRivers = new ArrayList<River>(count);

            for (int i = 0; i < count; i++) {
                Node tmp = rivers.item(i);

                String name = XMLUtils.xpathString(
                    tmp, "@art:name", ArtifactNamespaceContext.INSTANCE);

                theRivers.add(new DefaultRiver(name));
            }

            return theRivers.toArray(new River[count]);
        }
        catch (ConnectionException ce) {
            logger.error(ce, ce);
        }

        throw new ServerException(ERROR_NO_RIVERS_FOUND);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
